local T, check_number = ...

local handler = {
	["commands"] = function(data)
		print("\127FFFFFF"..T.help["commands"])
	end,
	
	["pos"] = function(data)
		if (T.hasmdi) then return end

		local x, y = data[1], data[2]
		local container = T.display_container
		if (check_number(x) and check_number(y)) then
			if (x:match("^[+-]")) then x = container.cx + x end
			if (y:match("^[+-]")) then y = container.cy + y end
-- 			container.cx, container.cy = tonumber(x), tonumber(y)
-- 			HUD.cboxlayer.REFRESH = "YES"
			T.x = x
			T.y = y
			T:SaveSettings()
			T:RebuildDisplay()

		else
			if (data[1] == "reset") then
				T.x = nil
				T.y = nil
				print("\127FFFFFFTargetlist position has been reset to default.")
				T:SaveSettings()
				T:RebuildDisplay()
			else
				local pos = container.cx.."x"..container.cy -- use the container positions since nil can't be concatenated
				if (not T.x) then pos = "unset (actual "..pos..")" end
				print("\127FFFFFFTargetlist position is "..pos)
			end
		end
	end,
	
	["fontscale"] = function(data)
		local newfontscale = check_number(data[1], T.defaults.fontscale.Lval, T.defaults.fontscale.Uval, T.defaults.fontscale.dp)
		if (newfontscale) then
			newfontscale = newfontscale < 0.1 and 0.1 or newfontscale
			T.fontscale = newfontscale
			print("\127FFFFFFFont size has been changed to "..newfontscale..".")
			T:SaveSettings()
			T:RebuildDisplay()
		else
			if (data[1] == "reset") then
				T.fontscale = T.defaults.fontscale.val
				print("\127FFFFFFThe font size has been been reset to "..T.fontscale..".")
				T:SaveSettings()
				T:RebuildDisplay()
			else
				print("\127FFFFFFThe current font size is "..T.fontscale..".")
			end
		end
	end,
	
	["help"] = function(data)
		local word = table.remove(data, 1)
		if (word) then
			word = string.lower(tostring(word))
			if (type(T.help[word]) == "string") then
				print("\127FFFFFF"..T.help[word])
			end

		else
			print("\127FFFFFF"..T.help["help"])
		end
	end,
              
	["highlight"] = function(data)
		local r = check_number(data[1], T.defaults.target_color.Lval, T.defaults.target_color.Uval)
		local g = check_number(data[2], T.defaults.target_color.Lval, T.defaults.target_color.Uval)
		local b = check_number(data[3], T.defaults.target_color.Lval, T.defaults.target_color.Uval)
		local a = check_number(data[4], T.defaults.target_color.Lval, T.defaults.target_color.Uval)
		if (r and g and b) then
			a = a or 65
			T.current_target_color = string.format("%d %d %d %d *", r, g, b, a)
			print("\127FFFFFFThe highlight color has been changed to "..T.current_target_color:match("(.*) %*"))
			T:SaveSettings()
		else
			if (data[1] == "reset") then
				T.current_target_color = T.defaults.target_color.val
				print("\127FFFFFFThe highlight color has been reset to "..T.current_target_color:match("(.*) %*"))
				T:SaveSettings()
			else
				print("\127FFFFFFThe current highlight color is "..T.current_target_color:match("(.*) %*"))
			end
		end
	end,
	
	["rows"] = function(data)
		local newrows = check_number(data[1], T.defaults.list_count.Lval)
		if (newrows) then
			T.list_count = newrows
			print("\127FFFFFFThe number of rows has been changed to "..newrows..".")
			T:SaveSettings()
			T:RebuildDisplay()
		else
			if (data[1] == "reset") then
				T.list_count = T.defaults.list_count.val
				print("\127FFFFFFThe number of rows has been been reset to "..T.list_count)
				T:SaveSettings()
				T:RebuildDisplay()
			else
				print("\127FFFFFFThe current number of rows is "..T.list_count)
			end
		end
	end,
	
	["width"] = function(data)
		local newwidth = check_number(data[1], T.defaults.name_width.Lval)
		if (newwidth) then
			T.name_width = newwidth
			print("\127FFFFFFName width has been changed to "..newwidth)
			T:SaveSettings()
			T:RebuildDisplay()

		else
			if (data[1] == "reset") then
				T.name_width = nil
				print("\127FFFFFFName width has been reset.")
				T:SaveSettings()
				T:RebuildDisplay()
			else
				local width = T.display[1][2][3][2].w
				if (not T.name_width) then width = "unset (actual "..width..")" end
				print("\127FFFFFFThe current name width is "..width..".")
			end
		end
	end,

	["options"] = function()
		ShowDialog(T.optionsdlg)
	end,
}

local function cmd_processor(_, data)
	if (not data) then
		handler["options"]()
-- 		print("\127FFFFFF"..T.help["help"])
		return
	end
	local cmd = string.lower(table.remove(data, 1))
	if (type(handler[cmd]) == "function") then handler[cmd](data) end
end

local function cycle_filter(_, data)
	if (data) then data = tonumber(data[1]) % 4 end
	T.filters:SetHighlight(T.filter + 1, false)
	T.filter = data or ((T.filter + 1) % 4)
	T.filters:SetHighlight(T.filter + 1, true)
	gkini.WriteInt(T.config_name, "Filter", T.filter)
end

local function cycle_faction_filter()
	T.faction_filter = (T.faction_filter + 1) % 4
	T.filters:SetHighlight(T.filter + 1, true)
	
end

local function change_target(i)
	if (T.shiplist[i]) then
		radar.SetRadarSelection(T.shiplist[i].nodeid, T.shiplist[i].objectid)
	end
end

local function target_next()
	local i = T.current_target_listindex or 0
	i = (i + 1) % T.list_count
	if (i == 0) then i = T.list_count end
	if (i > #T.shiplist) then i = 1 end
	change_target(i)
end

local function target_prev()
	local i = T.current_target_listindex or 0
	i = (i - 1) % T.list_count
	if (i == 0) then i = T.list_count end
	if (i > #T.shiplist) then i = #T.shiplist end
	change_target(i)
end

local verb_color = {[true] = "\127FF0000", [false] = "\12700FF00"}

local verb = {
	{[true] = "only Hive", [false] = "all NPCs"},
	{[true] = "HIDING", [false] = "SHOWING"}, -- reversed logic because the variable is hide_turrets
	{[true] = "NPC ship types", [false] = "NPC names"},
	{[true] = "INVERTED", [false] = "NORMAL"},
}
local function toggle_hive_only()
	T.show_hive_only = not T.show_hive_only
	print(string.format("\127FFFFFFTargetList is now showing %s%s\127o on the NPC filter.", verb_color[T.show_hive_only], verb[1][T.show_hive_only]))
	T:SaveSettings()
end

local function toggle_turrets()
	T.hide_turrets = not T.hide_turrets
	print(string.format("\127FFFFFFTargetList is now %s%s\127o turrets.", verb_color[T.hide_turrets], verb[2][T.hide_turrets]))

	T:SaveSettings()
end

local function toggle_npc_ship_types()
	T.show_npc_ship_types = not T.show_npc_ship_types
	print(string.format("\127FFFFFFTargetList is now showing %s%s\127o.", verb_color[not T.show_npc_ship_types], verb[3][T.show_npc_ship_types]))
	T:SaveSettings()
end

local function invert_faction_filter()
	T.invert_faction_filter = not T.invert_faction_filter
	print(string.format("\127FFFFFFTargetList faction filter is now %s%s\127o.", verb_color[T.invert_faction_filter], verb[4][T.invert_faction_filter]))
	iup.GetBrother(T.filters).bgcolor = "255 0 0 "..(T.invert_faction_filter and 35 or 0)
	T:SaveSettings()
end

local function set_key_binds()
	for i = 1, T.list_count < 10 and T.list_count or 10 do
		gkinterface.GKProcessCommand("bind "..(i % 10).." target"..i)
	end
	gkinterface.GKProcessCommand("bind - cyclefilter")
	gkinterface.GKProcessCommand("bind = cyclefactionfilter")
end

for i = 1, T.list_count < 10 and T.list_count or 10 do
	RegisterUserCommand("target"..i, change_target, i)
end
RegisterUserCommand("cyclefilter", cycle_filter)
RegisterUserCommand("cyclefactionfilter", cycle_faction_filter)
RegisterUserCommand("invertfactionfilter", invert_faction_filter)
RegisterUserCommand("targetnext", target_next)
RegisterUserCommand("targetprev", target_prev)
RegisterUserCommand("togglehiveonly", toggle_hive_only)
RegisterUserCommand("toggleturrets", toggle_turrets)
RegisterUserCommand("togglenpcshiptypes", toggle_npc_ship_types)
RegisterUserCommand("settargetbinds", set_key_binds)
RegisterUserCommand("targetlist", cmd_processor)

return change_target
